import {Component, EventEmitter, Input, Output} from '@angular/core';
import {DatePipe} from "@angular/common";
import {addDays} from "@progress/kendo-date-math";
import {MatButton} from "@angular/material/button";
import {MatIcon} from "@angular/material/icon";
import {MatBadge} from "@angular/material/badge";

export enum AdditionalText {
  Group,
  Professor,
  LectureHall,
  Other
}

@Component({
  selector: 'app-table-header',
  standalone: true,
  imports: [
    MatBadge,
    MatIcon,
    MatButton,
    DatePipe
  ],
  templateUrl: './table-header.component.html',
  styleUrl: './table-header.component.css'
})

export class TableHeaderComponent {
  protected additionalText: string = '';
  protected dataText: string = '';

  @Input() startWeek!: Date;
  @Input() currentWeek!: number;
  @Output() weekEvent: EventEmitter<boolean | null> = new EventEmitter<boolean | null>();

  AdditionalText(tab: AdditionalText = AdditionalText.Other, data: string | null = null) {
    if (data !== null) {
      this.dataText = data;
    }

    switch (tab) {
      case AdditionalText.Group:
        this.additionalText = 'для группы';
        break;
      case AdditionalText.LectureHall:
        this.additionalText = 'в аудитории';
        break;
      case AdditionalText.Professor:
        this.additionalText = 'для преподавателя';
        break;
    }
  }

  protected canBefore(): boolean {
    return this.currentWeek > 1;
  }

  protected readonly addDays = addDays;
}
